import type { UUID } from '@/application/types'

import { BaseEvent } from '@/application/base/baseEvent'
import { SocketEvent } from '@/application/enums'
import CharacterTypeRepository from '@/repositories/characterTypeRepository'

interface IPayload {
  id: UUID
}

export default class CharacterTypeDeleteEvent extends BaseEvent {
  public listen(): void {
    this.socket.on(SocketEvent.GM_CHARACTERTYPE_REMOVE, this.handleEvent.bind(this))
  }

  private async handleEvent(data: IPayload, callback: (response: boolean) => void): Promise<void> {
    try {
      if (!(await this.isCharacterGM())) return

      const characterTypeRepository = new CharacterTypeRepository()
      const characterType = await characterTypeRepository.getById(data.id)
      if (!characterType) return callback(false)

      await characterType.delete()
      return callback(true)
    } catch (error) {
      this.logger.error(`Error deleting character type ${data.id}: ${error instanceof Error ? error.message : String(error)}`)
      return callback(false)
    }
  }
}
